//Distributed under Kellen's sort-of-free license.
//http://people.colgate.edu/kmyers/license.html
//(c) Kellen Myers and Joseph Parrish 2007

typedef struct {
	// the highest value of last_bit before we must allocate more space.
	int max_bit;
	// the highest valid number in the coloring.
	int last_bit;
	// the bit-string representing this two-coloring.
	unsigned char *bits;
} coloring_t;

typedef struct {
	// absolute bit number within the bit-string.
	int abs_bit;
	// the highest valid absolute bit we may access.
	int max_bit;
	// a flag indicating that this iterator has rolled over to start.
	int overflow;
	// a flag indicating that this iterator is not to be moved.
	int held;
} iterator_t;

// returns the position of this iterator.
int iter_pos(iterator_t *iter);
// reset all iterators.
void iter_reset(iterator_t *iters, int num_vars, coloring_t *c);
// hold one iterator at the final position.
void iter_hold(iterator_t *iter);

// allocate a new coloring with at least the given number of bytes of storage.
coloring_t *color_new(int n);
// read the color of the given number.  non-zero means blue.
int color_read(coloring_t *c, int n);
// grow the coloring by adding a red.
void color_grow(coloring_t *c);
// backtrack the coloring to the last red, make it blue.
void color_backtrack(coloring_t *c);
// restart the coloring at {r}.
void color_restart(coloring_t *c);
// free the allocated coloring.
void color_free(coloring_t *c);
// copy the coloring from src into dst.
// NOTE: if dst->bits is non-null, it is treated as a valid buffer.
void copy_coloring(coloring_t *dst, coloring_t *src);
// fill in coloring randomly.
// NOTE: if dst->bits is non-null, it is treated as a valid buffer.
void color_randomly(coloring_t *c, int min_bit, int max_bit);

// move iterators to the next potential monochromatic solution.
int next(coloring_t *c, iterator_t *iters, int x, int num_vars);
// fill in the variable array from the iterator positions.
void fill_vars(iterator_t *iters, int *var_array, int num_vars);

