//Distributed under Kellen's sort-of-free license.
//http://people.colgate.edu/kmyers/license.html
//(c) Kellen Myers and Joseph Parrish 2007

#include <string>
using std::string;

#define lower(x) ((x) | 0x20)
#define ltonum(x) (lower(x) - 'a')
#define between(x, l, u) ((x) >= (l) && (x) <= (u))

static char *eat_integer(char *start, int *n)
{
	char *end;
	int x;
	x = strtol(start, &end, 10);
	if (n) *n = x;
	return end;
}

extern "C" char *translate_eq(char *human_string)
{
	int nextvar = 0;
	bool varwait = false, expnext = false;
	string aterms, mterms, vterm, full;
	int mterm_count = 0;
	int aterm_count = 0;
	int varnums[26];
	int vargood[26];
	int j, num;
	char numstring[32];

	for (j = 0; j < 26; j++) {
		vargood[j] = 0;
	}

	while (*human_string) {
		if (between(*human_string, '0', '9')) {
			int n;
			human_string = eat_integer(human_string, &n);
			// so that we advance one later
			// and end up on the next token....
			human_string--;
			snprintf(numstring, 32, "%d", n);
			if (expnext) {
				// add exponent.
				mterms += numstring;
				mterms += '*';
				expnext = false;
			} else {
				if (varwait) {
					// flush variable.
					mterms += vterm;
					mterms += "1*";
					varwait = false;
					mterm_count++;
				}
				// add constant.
				mterms += "0.";
				mterms += numstring;
				mterms += '*';
				mterm_count++;
			}
		} else if (between(lower(*human_string), 'a', 'z')) {
			if (expnext) {
				// FIXME: no variables as exponents.
				printf("No variables as exponents. (1)\n");
				return NULL;
			} else if (varwait) {
				// flush variable.
				mterms += vterm;
				mterms += "1*";
				varwait = false;
				mterm_count++;
			}
			// calculate variable number.
			if (vargood[ltonum(*human_string)]) {
				num = varnums[ltonum(*human_string)] + 1;
			} else {
				varnums[ltonum(*human_string)] = nextvar;
				vargood[ltonum(*human_string)] = 1;
				nextvar++;
				num = nextvar;
			}
			snprintf(numstring, 32, "%d", num);
			vterm = numstring;
			vterm += '.';
			varwait = true;
		} else if (*human_string == '*') {
			// do nothing.  multiplication is implied.
		} else if (*human_string == '^') {
			if (!varwait) {
				// FIXME: no orphan exponents.
				printf("No orphan exponents. (2)\n");
				return NULL;
			} else if (expnext) {
				// FIXME: no double exponents.
				printf("No double exponents. (3)\n");
				return NULL;
			}
			mterms += vterm;
			expnext = true;
			varwait = false;
			mterm_count++;
		} else if (*human_string == '+') {
			if (varwait) {
				// flush variable.
				mterms += vterm;
				mterms += "1*";
				varwait = false;
				mterm_count++;
			}
			if (!mterm_count) {
				printf("'%s'\n", human_string);
				// FIXME: no double-plus.
				printf("No double-plus. (4)\n");
				return NULL;
			} else if (expnext) {
				// FIXME: no dangling exponent.
				printf("No dangling exponents. (5)\n");
				return NULL;
			}
			snprintf(numstring, 32, "%d", mterm_count);
			aterms += numstring;
			aterms += ':';
			aterms += mterms;
			aterms += ',';
			mterms = "";
			mterm_count = 0;
			aterm_count++;
		} else if (*human_string == '-') {
			if (varwait) {
				// flush variable.
				mterms += vterm;
				mterms += "1*";
				varwait = false;
				mterm_count++;
			}
			if (!mterm_count) {
				// FIXME: no plus a negative.
				printf("No plus negative. (6)\n");
				return NULL;
			} else if (expnext) {
				// FIXME: no dangling exponent.
				printf("No dangling exponent. (7)\n");
				return NULL;
			}
			snprintf(numstring, 32, "%d", mterm_count);
			aterms += numstring;
			aterms += ':';
			aterms += mterms;
			aterms += ',';
			mterms = "";
			mterm_count = 0;
			aterm_count++;
			// add constant -1.
			mterms += "0.-1*";
			mterm_count++;
		} else if (*human_string == ' ') {
			// do nothing.
		} else {
			printf("Invalid character 0x%02x. (8)\n", *human_string);
			return NULL;
		}
		human_string++;
	}

	if (varwait) {
		// flush variable.
		mterms += vterm;
		mterms += "1*";
		varwait = false;
		mterm_count++;
	}
	if (!mterm_count) {
		// FIXME: no plus/minus nothing.
		printf("No plus nothing. (9)\n");
		return NULL;
	} else if (expnext) {
		// FIXME: no dangling exponent.
		printf("No dangling exponent. (10)\n");
		return NULL;
	}
	snprintf(numstring, 32, "%d", mterm_count);
	aterms += numstring;
	aterms += ':';
	aterms += mterms;
	aterms += ',';
	aterm_count++;

	if (!nextvar) nextvar++;
	snprintf(numstring, 32, "%d", nextvar);
	full = numstring;
	full += ']';
	snprintf(numstring, 32, "%d", aterm_count);
	full += numstring;
	full += ';';
	full += aterms;

	return (char *)strdup(full.c_str());
}

