//Distributed under Kellen's sort-of-free license.
//http://people.colgate.edu/kmyers/license.html
//(c) Kellen Myers and Joseph Parrish 2007

// vim:set ts=4:
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef WIN32
# include <windows.h>
#else
# include <unistd.h>
# include <sys/resource.h>
#endif

#include "colorings.h"
#include "eq.h"
#include "stats.h"
#include "rado.h"

// global, see global.h
int status;
// rado info struct.
static rado_info_t ri;

// phony front-end check for self-contained app.
// global, see global.h
int check_front(int fd, coloring_t *c, rado_info_t *ri)
{
	static unsigned int x = 0;

	x++;
	if (x & 0xff) return 0;

	fprintf(stderr, "\r%d colorings checked; peak length = %d; (est) %.3f%% space searched.",
		x, ri->max_len, color_progress(c));
	fflush(stderr);

	return 0;
}

#define basename0(x) (strrchr(x, '\\') ? strrchr(x, '\\') + 1 : x)
#define basename(x) (strrchr(x, '/') ? strrchr(x, '/') + 1 : basename0(x))

int main(int argc, char **argv)
{
	char *string;
	int i, stop;
	coloring_t *c;

	if ((argc < 2) || (argc > 3)) {
		printf("Usage: %s Equation [Stop]\n", basename(argv[0]));
		return 1;
	}

	string = translate_eq(argv[1]);
	if (!string) {
		printf("Invalid equation specified.\n");
		return 1;
	}

	if (argc >= 3) {
		stop = atoi(argv[2]);
	} else {
		stop = 0x7fffffff;
	}

	ri.colorings_allocd = 1024;
	ri.colors_at_max = (coloring_t *)malloc(sizeof(coloring_t) * ri.colorings_allocd);
	for (i = 0; i < ri.colorings_allocd; i++) {
		ri.colors_at_max[i].bits = NULL;
	}

	// set low priority.
#ifdef WIN32
	SetPriorityClass(GetCurrentProcess(), IDLE_PRIORITY_CLASS);
#else
	setpriority(PRIO_PROCESS, getpid(), 19);
#endif

	c = color_new(1);
	color_restart(c);
	status = 3;
	rado(-1, string, c, stop, &ri, 1);
	fprintf(stderr, "\r                                                                           \r");
	fflush(stderr);
	if (!ri.done) {
		printf("Search stopped prematurely at len %d.\n", ri.max_len);
		return 0;
	}

	printf("Done searching, max len=%d, %d valid at len=%d.\n",
		ri.max_len, ri.valid_at_max, ri.max_valid_len);

	for (i = 0; i < ri.valid_at_max; i++) {
		color_dump(&ri.colors_at_max[i]);
	}

	return 0;
}

